# Copyright 2009, Red Hat, Inc
# Copyright 2009, Yaakov Nemoy
#
# This software may be freely redistributed under the terms of the GNU
# general public license, version 2 or higher.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
from __future__ import with_statement

import re

from contextlib import contextmanager
from htmlentitydefs import name2codepoint
from os import chdir, getcwd


def htmlentitydecoder(s):
    # first we scrub all named entities into normalacy.
    s1 = re.sub('&(%s);' % '|'.join(name2codepoint),
	lambda m: unichr(name2codepoint[m.group(1)]), s)
    #then we scrub the ones that are numeric instead
    return re.sub('&#(\d*?);', lambda m: unichr(int(m.group(1))), s1)

@contextmanager
def pwd(dir):
    old_dir = getcwd()
    chdir(dir)
    yield
    chdir(old_dir)

def iter_in_transaction(session, iter):
    with session.begin(subtransactions=True):
        for item in iter:
            yield item
#             session.flush()
#             session.commit()
        session.flush()
__all__ = ['htmlentitydecoder', 'pwd', 'iter_in_transaction']



# Copyright 2009, Red Hat, Inc
# Copyright 2009, Yaakov Nemoy
#
# This software may be freely redistributed under the terms of the GNU
# general public license, version 2 or higher.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
from __future__ import with_statement

import re
import copy
import codecs
import gzip
import mailbox
import dateutil
import datetime

import dateutil.parser as dup
import httplib as http
import urlparse as up
import urlgrabber as ug

from urlgrabber.grabber import URLGrabber
from re import compile
from os import makedirs
from os.path import join

import sqlalchemy
import sqlalchemy.orm.attributes

from sqlalchemy import create_engine, Table, MetaData, Table, Column,\
    Integer, Text, DateTime
from sqlalchemy.exceptions import InvalidRequestError
from sqlalchemy.orm import mapper, sessionmaker, scoped_session, relation
from sqlalchemy.schema import ForeignKey, UniqueConstraint
from sqlalchemy.ext.declarative import declarative_base, DeclarativeMeta

from util import iter_in_transaction, pwd, htmlentitydecoder

engine = None #create_engine('sqlite:///sqllite.db') #, echo=True)
metadata = MetaData()
Session = scoped_session(sessionmaker(autoflush=False))#, bind=engine))

class Hell(Exception): pass

def get_engine(dburi=None):
    global engine
    if engine:
        return engine
    elif dburi:
        engine = create_engine(dburi) #, echo=True)
        Session.configure(bind=engine)
        return engine
    else:
        raise Hell


stream_classes = dict()

def register_stream(cls, name):
    global stream_classes
    stream_classes[name] = cls


class MetaSqlBase(DeclarativeMeta):
    def __init__(cls, name, bases, attrs):
        DeclarativeMeta.__init__(cls, name, bases, attrs)
        sql_attrs = dict()
        unique_sql_attrs = dict()
        for attr in dir(cls):
            attr_obj = getattr(cls, attr)
            if isinstance(attr_obj, Column) \
                    or isinstance(attr_obj,
                                  sqlalchemy.orm.attributes.InstrumentedAttribute) \
                    or getattr(attr_obj, "_sql_attribute", False):
                sql_attrs[attr] = True
        unique_sql_attrs = cls.define_unique(unique_sql_attrs)
        cls._validate_remote = cls._clear_inv_keys_func(sql_attrs)
        cls._unique_remote = cls._clear_inv_keys_func(unique_sql_attrs)

    def _clear_inv_keys_func(self, v_keys):

        def clear_inv_keys(remote):
            v_remote = copy.copy(remote)
            for key in remote.iterkeys():
                if key not in v_keys:
                    del v_remote[key]
            return v_remote
        return staticmethod(clear_inv_keys)


class SqlBase(object):
    __metaclass__ = MetaSqlBase
    _decl_class_registry = dict()

    @classmethod
    def create_or_update(cls, **remote):
        try:
            sql_obj = session.query(cls)\
                .filter_by(**cls._unique_remote(remote))\
                .one()
            if sql_obj.needs_update(remote):
                sql_obj.update_props(**cls._validate_remote(remote))
                sql_obj.update(remote)
        except InvalidRequestError, e:
            sql_obj = cls(**cls._validate_remote(remote))
            session.add(sql_obj)
            sql_obj.update(remote)
        return sql_obj

    def update_props(self, **keys):
        self.__dict__.update(keys)

    def needs_update(self, remote):
        raise NotImplementedError

    def update(self, remote):
        raise NotImplementedError

    @staticmethod
    def define_unique(d):
        return d

Base = declarative_base(metadata=metadata, cls=SqlBase, metaclass=MetaSqlBase)


class MetaStream(type(Base)):
    def __init__(cls, name, bases, attrs):
        source = name.lower()
        cls.source = source
        type(Base).__init__(cls, name, bases, attrs)
        register_stream(cls, source)


class Stream(Base):
    __metaclass__ = MetaStream
    __tablename__ = 'streams'
    id = Column('id', Integer, primary_key=True)
    type = Column('type', Text, nullable=False)
    name = Column('name', Text)
    __mapper_args__ = dict(polymorphic_on = type, polymorphic_identity="stream")

    @property
    def source_class(self):
        return Source

    @staticmethod
    def define_unique(d):
        Base.define_unique(d)
        d['type'] = True
        d['name'] = True
        return d

    def needs_update(self, remote):
        return True

    def remote_iter(self):
        raise NotImplementedError

    @property
    def cache(self):
        return join(self.source, self.name)

    def update(self, remote):
        with pwd(CACHE_DIR):
            try:
                makedirs(self.cache)
            except OSError, e:
                print e
            remotes = self.remote_iter()
            for remote in iter_in_transaction(session, remotes):
                self.source_class.create_or_update(**remote)


class Mailman(Stream):
    __tablename__ = 'mailmen'
    id = Column('id', Integer, ForeignKey('streams.id'), primary_key=True)
    __mapper_args__ = dict(polymorphic_identity='mailman', inherits=Stream)

    @property
    def archive(self):
        raise NotImplementedError

    @property
    def listinfo(self):
        raise NotImplementedError

    def url(self, month):
        raise NotImplementedError

    def mbox(self, month):
        print 'deprecated usage'
        return self.url(month)

    def cached_source(self, month):
        return join(self.cache, month)

class RHMailman(Mailman):
    __tablename__ = 'rhmailmen'
    id = Column('id', Integer, ForeignKey('mailmen.id'), primary_key=True)
    __mapper_args__ = dict(polymorphic_identity='rhmailman', inherits=Mailman)

    regex = compile(r"""<a href=\"(?P<url>(?P<year>\d{4})-(?P<month>\w*?)/thread.html)\">\[ Thread \]""", re.I | re.S)
    def remote_iter(self):
        conn = URLGrabber(reget='check_timestamp',
                          user_agent='jemoeder/2.0',
                          prefix='http://www.redhat.com',
                          retry=0)

        site = self.archive
        html = ug.urlopen(site).read()
        matches = self.regex.finditer(html)
        match_dicts = (match.groupdict() for match in matches)

        for match_dict in match_dicts:
            match_dict['cache_url'] = self.url(match_dict['url'])
            match_dict['cache_file'] = match_dict['url'].replace('/', '-')
            match_dict['identifier'] = "%s-%s" % ( match_dict['year'], match_dict['month'])
            match_dict['stream'] = self
            match_dict['month'] = dup.parse('%s %s' % (match_dict['month'], match_dict['year']))

            resp = conn.urlread(up.urlsplit(match_dict['cache_url']).path)
            match_dict['size'] = len(resp)
            yield match_dict

    @property
    def source_class(self):
        return RHMailmanSource

    @property
    def archive(self):
        return 'https://www.redhat.com/archives/%s/' % self.name

    @property
    def listinfo(self):
        return 'https://www.redhat.com/mailman/listinfo/%s/' % self.name

    def url(self, month):
        return self.archive + month


class MBoxMailman(Mailman):
    __tablename__ = 'mboxmailmen'
    id = Column('id', Integer, ForeignKey('mailmen.id'), primary_key=True)
    __mapper_args__ = dict(polymorphic_identity='mboxmailman')

    @property
    def source_class(self):
        return MBoxSource

    def remote_iter(self):
        site = self.archive
        html = ug.urlopen(site).read()
        matches = self.regex.finditer(html)
        match_dicts = (match.groupdict() for match in matches)
        for match_dict in match_dicts:
            match_dict['cache_url'] = self.url(match_dict['mbox'])
            match_dict['cache_file'] = match_dict['mbox']
            match_dict['identifier'] = match_dict['mbox']
            match_dict['stream'] = self
            match_dict['month'] = dup.parse('%s %s' % (match_dict['month'], match_dict['year']))
            if type(match_dict['size']) is str:
                match_dict['size'] = int(match_dict['size'])
            yield match_dict


class FHMailman(MBoxMailman):
    __tablename__ = 'fhmailman'
    id = Column('id', Integer, ForeignKey('mboxmailmen.id'), primary_key=True)
    __mapper_args__ = dict(polymorphic_identity='fhmailman')

    regex = compile(r"""<td>(?P<month>\w*?) (?P<year>\d{4}):</td>.*?<A href="(?P<mbox>(?P=year)-(?P=month).txt.gz)".*?text (?P<size>\d.*?) (KB|bytes)""", re.I | re.S)

    @property
    def archive(self):
        return 'https://fedorahosted.org/pipermail/%s/' % self.name

    @property
    def listinfo(self):
        return 'https://fedorahosted.org/mailman/listinfo/%s/' % self.name

    def url(self, month):
        return self.archive + month

class FPMailman(MBoxMailman):
    __tablename__ = 'fpmailmen'
    id = Column('id', Integer, ForeignKey('mboxmailmen.id'), primary_key=True)
    __mapper_args__ = dict(polymorphic_identity='fpmailman')

    regex = compile(r"""<td>(?P<month>\w*?) (?P<year>\d{4}):</td>.*?<A href="(?P<mbox>(?P=year)-(?P=month).txt.gz)".*?text (?P<size>\d.*?)(KB|bytes)""", re.I | re.S)

    @property
    def archive(self):
        return 'http://lists.fedoraproject.org/pipermail/%s/' % self.name

    @property
    def listinfo(self):
        return 'https://admin.fedoraproject.org/mailman/listinfo/%s' % self.name

    def url(self, month):
        return self.archive + month


class Source(Base):
    __tablename__ = 'sources'
    id         = Column('id', Integer, primary_key=True)
    type = Column('type', Text, nullable=False)
    cache_file = Column('cache_file', Text)
    cache_url  = Column('cache_url', Text)
    size       = Column('size', Integer)
    identifier = Column('identifier', Text)
    month      = Column('month', DateTime)
    stream_id  = Column('stream_id', Integer,
                        ForeignKey('streams.id'), index=True)
    stream     = relation(Stream, primaryjoin=stream_id==Stream.id,
                                 backref='sources', order_by=month)
    __mapper_args__ = dict(polymorphic_on = type, polymorphic_identity="source")

    @property
    def fact(self):
        return Fact

    @staticmethod
    def define_unique(d):
        Base.define_unique(d)
        d['identifier'] = True
        d['stream'] = True
        return d

    @property
    def cache_location(self):
        return join(self.stream.cache, self.cache_file)

    @property
    def cache_dir(self):
        return join(self.stream.cache, self.identifier)

    def needs_update(self, remote):
        return not (self.size == remote['size'])

    def cache_source(self, url):
        location = self.cache_location
        url_loc = ug.urlgrab(url, filename=location)

    def update(self, remote):
        self.cache_source(remote['cache_url'])


class MBoxSource(Source):
    __tablename__ = 'mbox_sources'
    id = Column('id', Integer, ForeignKey('sources.id'), primary_key=True)
    __mapper_args__ = dict(polymorphic_identity='mbox_source')

    @property
    def fact(self):
        return MBoxFact

    def update(self, remote):
        self.cache_source(remote['cache_url'])

        mbox = self.mbox
        for email in iter_in_transaction(session, mbox):
            email = dict(email.items())
            try:
                email['message_id'] = email['Message-ID']
                email['sender'] = email['From']
                email['sender_domain'] = self.fact.domain_from_sender(email['sender'])
                email['source'] = self
                # sometimes these fields are blank, which is kinda ok, because it's a garbage message
                # but dateutil can't handle None, so we have to use this hack
                try:
                    email['date'] = dup.parse(email['Date'] or str(datetime.datetime.min)).replace(day=1)
                except ValueError:
                    print "TODO: Fix bad date parsing!"
                    email['date'] = datetime.datetime.min
                email['subject'] = email['Subject']
                self.fact.create_or_update(**email)
            except KeyError:
                continue

    #TODO: This needs to be abstracted a bit
    @property
    def mbox(self):
        with pwd(self.stream.cache):
            path_in = self.cache_file
            try:
                f_in = gzip.open(path_in, 'rb')
                path_out = path_in.replace('.gz', '')
                f_out = open(path_out, 'wb')
                f_out.writelines(f_in)
            except IOError, e:
                return mailbox.mbox(path_in)
            finally:
                f_out.close()
                f_in.close()
            return mailbox.mbox(path_out)


class RHMailmanSource(Source):
    __tablename__ = 'rhmailmainsources'
    id = Column('id', Integer, ForeignKey('sources.id'), primary_key=True)
    __mapper_args__ = dict(polymorphic_identity='rhmailmansource')

    regex = compile(r'''<li><strong><a name="\d*" href="(?P<message>msg(?P<number>\d*).html)">.*?<em>(?P<name>.*?)</em>''', re.I | re.S)
    regex_from = compile(r'''<li><em>From</em>: (?P<from>.*?)</li>''', re.I)
    regex_date = compile(r'''<li><em>Date</em>: (?P<date>.*?)</li>''', re.I)
    regex_message_id = compile(r'''<!--X-Message-Id: (?P<message_id>.*?) -->''', re.I)
    regex_subject = compile(r'''<!--X-Subject: (?P<subject>.*?) -->''', re.I)

    def __init__(self, **keys):
        Source.__init__(self, **keys)
        try:
            makedirs(self.cache_dir)
        except OSError, e:
            print e

    @property
    def fact(self):
        return RHMailmanFact

    def update(self, remote):
        self.cache_source(remote['cache_url'])

        conn = URLGrabber(reget='check_timestamp',
                          user_agent='jemoeder/2.0',
                          prefix='http://www.redhat.com',
                          retry=0)

        month_page = codecs.open(self.cache_location, encoding='utf_8').read()
        matches = self.regex.finditer(month_page)
        emails = (match.groupdict() for match in matches)
        for email in emails:
            url = self.stream.archive + self.identifier + '/' + email['message']
            loc = join(self.stream.cache, self.identifier, email['message'])

            url_split = up.urlsplit(url)
            url_path = url_split.path
            print url_path
            email_data = conn.urlread(url_path)
            print len(email_data)
            with file(loc, 'w') as f:
                f.write(email_data)

#             url_loc = urlretrieve(url, loc)
#             email_data = htmlentitydecoder(codecs.open(loc, encoding='utf_8').read())

            try:
                email['sender'] = self.regex_from.search(email_data).group('from')
            except AttributeError:
                email['sender'] = 'nobody@example.com'
            email['date'] = self.regex_date.search(email_data).group('date')
            email['message_id'] = self.regex_message_id.search(email_data).group('message_id')
            email['subject'] = self.regex_subject.search(email_data).group('subject')
            email['source'] = self
            try:
                email['date'] = dup.parse(email['date'])
            except ValueError:
                print "TODO: Fix bad date parsing!"
                email['date'] = datetime.datetime.min
            self.fact.create_or_update(**email)



class Fact(Base):
    __tablename__ = 'facts'
    id         = Column('id', Integer, primary_key=True)
    type = Column('type', Text, nullable=False)
    sender_name = Column('sender_name', Text, index=True)
    sender_handle = Column('sender_handle', Text, index=True)
    sender     = Column('sender', Text, index=True)
    sender_domain = Column('sender_domain', Text, index=True)
    message_id = Column('message_id', Text, index=True)
    date       = Column('date', DateTime)
    subject    = Column('subject', Text)
    source_id  = Column('source_id', Integer,
                        ForeignKey('sources.id'), index=True)
    source     = relation(Source, primaryjoin=source_id==Source.id,
                                 backref='facts')
    __mapper_args__ = dict(polymorphic_on = type, polymorphic_identity="fact")

    @staticmethod
    def define_unique(d):
        Base.define_unique(d)
        d['sender'] = True
        d['message_id'] = True
        d['source'] = True
        return d

    def needs_update(self, remote):
        return False

    def update(self, remote):
        return None

    @classmethod
    def domain_from_sender(cls, sender):
        raise NotImplementedError


class MBoxFact(Fact):
    __tablename__ = 'mboxfacts'
    id = Column('id', Integer, ForeignKey('facts.id'), primary_key=True)
    __mapper_args__ = dict(polymorphic_identity='mboxfact')

    domain_regex = re.compile(r'.*at (.*?) \(.*\)')

    @classmethod
    def domain_from_sender(cls, sender):
        return cls.domain_regex.findall(sender)[0]

class RHMailmanFact(Fact):
    __tablename__ = 'rhmailmanfacts'
    id = Column('id', Integer, ForeignKey('facts.id'), primary_key=True)
    __mapper_args__ = dict(polymorphic_identity='rhmailmanfact')

    domain_regex = re.compile(r"""(?P<name>[^<]*?)@(?P<domain>[^>]*)>?""")
    domain_regex2 = re.compile(r"""(?P<name>.*) at (?P<domain>\S*)""")

    @classmethod
    def domain_from_sender(cls, sender):
        try:
            return cls.domain_regex.search(sender).group('domain')
        except AttributeError, e:
            return cls.domain_regex2.search(sender).group('domain')


session = Session()

__all__ = ['Stream', 'FPMailman', 'FHMailman', 'RHMailman', 'Source',
           'Fact', 'session', 'Session', 'metadata', 'engine',
           'stream_classes', 'get_engine']



# Copyright 2009, Red Hat, Inc
# Copyright 2009, Yaakov Nemoy
#
# This software may be freely redistributed under the terms of the GNU
# general public license, version 2 or higher.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from configobj import ConfigObj

from migrate.versioning.shell import main

import ekg.util
import ekg.model
import ekg.migration

from ekg.grapher import write_graph_page
from ekg.model import get_engine

UPDATE_ALL = True
CACHE_DIR = '.'

commands = list()

def command(f):
    global commands
    commands.append(f)
    return f


@command
def generate_tables():
    config = ConfigObj('settings.ini')
    db = config['dburi']
    engine = get_engine(db)
    ekg.model.metadata.create_all(engine)


@command
def scan():
    config = ConfigObj('settings.ini')
    global CACHE_DIR
    global UPDATE_ALL
    CACHE_DIR = config['cache_dir']
    UPDATE_ALL = config['update_all']
    ekg.model.UPDATE_ALL = UPDATE_ALL
    ekg.model.CACHE_DIR = CACHE_DIR
    for list, source \
            in ekg.util.iter_in_transaction(session, config['lists'].items()):
        StreamClass = stream_classes[source]
        stream = StreamClass.create_or_update(name=list)
    session.flush()
    session.commit()


@command
def graph():
    write_graph_page('001')


@command
def migrate():
    pth = ekg.migration.__path__[0]
    config = ConfigObj('settings.ini')
    db = config['dburi']
    main(url=db, repository=pth)


@command
def create_db():
    pth = ekg.migration.__path__[0]
    config = ConfigObj('settings.ini')
    db = config['dburi']
    engine = get_engine(db)
    con = engine.connect()
    con.close()
    main(['version_control'], url=db, repository=pth)


@command
def upgrade():
    pth = ekg.migration.__path__[0]
    config = ConfigObj('settings.ini')
    db = config['dburi']
    main(['upgrade'], url=db, repository=pth)


@command
def setup():
    create_db()
    upgrade()


__all__ = ['generate_tables', 'scan', 'graph', 'migrate', 'upgrade', 'setup', 'create_db']



# Copyright 2009, Red Hat, Inc
# Copyright 2009, Yaakov Nemoy
#
# This software may be freely redistributed under the terms of the GNU
# general public license, version 2 or higher.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
from __future__ import with_statement

import datetime
import dateutil.parser
import gzip
import itertools
import mailbox

from configobj import ConfigObj
from dateutil.relativedelta import relativedelta
from os import makedirs
from os.path import join
from urllib import urlopen, urlretrieve

from sqlalchemy import *
from sqlalchemy.exceptions import InvalidRequestError

import model
import util

from model import *

config = ConfigObj('settings.ini')

def domain_counts_for_source(source):
    counts = session.query(func.count(Fact.sender_domain),
                         Fact.sender_domain)\
        .filter_by(source=source)\
        .group_by(Fact.sender_domain)\
        .order_by(func.count(Fact.sender_domain).desc())\
        .all()
    to_return = dict()
    for count, domain in counts:
        to_return[domain] = count
    return to_return
        

def sources_in_order_for_stream(stream):
    return session.query(Source)\
        .filter_by(stream=stream)\
        .order_by(Source.month)

def month_w_count(stream):
    for source in sources_in_order_for_stream(stream):
        yield (source.month, domain_counts_for_source(source))

def do_streams():
    for stream in session.query(Stream).all():
        pass

def do_month(month, year):
    for source in session.query(Source)\
            .filter_by(month=dateutil.parser.parse("%s %s" % (month, year))\
                           .replace(day=1))\
            .all():
        print domain_counts_for_source(source)

def periodic_totals(months, predicate):
    ytd = dict()
    first_month = None
    for month, counts in months:
        if not first_month:
            first_month = start_year_of(month)
        if predicate(month):
            if ytd == dict():
                pass
            else:
                yield (first_month, ytd)
                ytd = dict()
                first_month = month
        for domain, count in counts.iteritems():
            try:
                ytd[domain] += count
            except KeyError:
                ytd[domain] = count
    if first_month:
        yield (first_month, ytd)
    else:
        raise StopIteration

def start_year_of(date):
    if is_start_year(date):
        return date
    else:
        new_month = date.month - 1
        new_year = date.year if new_month else date.year - 1
        new_month = new_month if new_month else 12
        return start_year_of(date.replace(month=new_month, year=new_year))

def is_start_year(date):
    return date.month == int(config['new_years_month'])

def is_start_quarter(date):
    return date.month % 3 == int(config['new_years_month']) % 3

def yearly_totals(months):
    return periodic_totals(months, is_start_year)

def quarterly_totals(months):
    return periodic_totals(months, is_start_quarter)

def pairwise(iterable):
    "s -> (s0,s1), (s1,s2), (s2, s3), ..."
    a, b = itertools.tee(iterable)
    try:
        b.next()
    except StopIteration:
        pass
    return itertools.izip(a, b)

def delta_counts(periods):
    for previous, current in pairwise(periods):
        prev_month, prev_counts = previous
        cur_month, cur_counts = current

        domains = set(prev_counts.keys()).union(cur_counts.keys())

        deltas = dict()
        for domain in domains:
            prev_count = prev_counts.get(domain, 0)
            cur_count = cur_counts.get(domain, 0)
            deltas[domain] = cur_count - prev_count
        yield prev_month, cur_month, deltas


def pairshift(i, j):
    i = iter(i)
    j = iter(j)
    yield i.next(), None
    for x in itertools.izip(i, j):
        yield x

#TODO: this is the wrong way to do strings
# I have a fine library for spitting out html that needs to be packaged in fedora first
def generate_period_table(mm, periods, deltas):
    yield "<table border=2>"
    yield "<tr><th>period</th><th>posts by domain</th><th>deltas since last period</th></tr>"
    for period, delta in pairshift(periods, deltas):
        to_yield = "<tr><td>" + period[0].strftime('%B %Y') + "</td><td><ul>"
        counts = period[1]
        if delta:
            delta_counts = delta[2]
        keys = sorted(counts, key=counts.get, reverse=True)
        for key in keys:
            to_yield += '<li>' + key + ' : ' + str(counts[key]) + '</li>'
        to_yield += '</ul></td><td><ul>'
        if delta:
            for key in keys:
                to_yield += '<li>' + key + ' : ' + str(delta_counts[key]) + '</li>'
            to_yield += '</ul></td></tr>'
        yield to_yield
    yield '</table>'

def generate_page(*iters):
    yield '<html><body>'
    for iter in iters:
        for i in iter:
            yield i
    yield '</body></html>'

def generate_mm_table(mm):
    month_periods = list(month_w_count(mm))
    month_deltas = list(delta_counts(month_periods))

    quarter_periods = list(quarterly_totals(month_periods))
    quarterly_deltas = list(delta_counts(quarter_periods))

    yearly_periods = list(yearly_totals(month_periods))
    yearly_deltas = list(delta_counts(yearly_periods))

    print 'mm'
    print mm.name, mm.type
    print

    print 'month'
    print month_periods
    print month_deltas
    print

    print 'quarter'
    print quarter_periods
    print quarterly_deltas
    print

    print 'year'
    print yearly_periods
    print yearly_deltas

    yield '<h1>' + mm.name + ' of type ' + mm.type + '</h1>'
    yield '<h2>monthly</h2>'
    for line in generate_period_table(mm, month_periods, month_deltas):
        yield line
        
    yield '<h2>quarterly</h2>'
    for line in generate_period_table(mm, quarter_periods, quarterly_deltas):
        yield line
        
    yield '<h2>yearly</h2>'
    for line in generate_period_table(mm, yearly_periods, yearly_deltas):
        yield line

def generate_all_tables():
    for mm in session.query(Stream).all():
        for line in generate_mm_table(mm):
            yield line

def generate_all_tables_page():
    for line in generate_page(generate_all_tables()):
        yield line
    

def write_graph_page(num):
    with file('graph.test.%s.html' % num, 'w') as f: 
        for line in generate_all_tables_page():
            f.write(line)

__all__ = ['domain_counts_for_source', 'sources_in_order_for_stream',
           'month_w_count', 'do_streams', 'do_month', 'periodic_totals',
           'start_year_of', 'is_start_year', 'is_start_quarter', 'yearly_totals',
           'quarterly_totals', 'pairwise', 'delta_counts', 'pairshift',
           'generate_page', 'generate_mm_table', 'generate_all_tables_page',
           'generate_all_tables', 'write_graph_page']



#!/usr/bin/env python

from setuptools import setup, find_packages

from ekg.commands import commands

def command_funcs(cs):
    return ['ekg-%s = %s:%s' % 
            (c.__name__.replace('_', '-'),
             c.__module__,
             c.__name__) for c in cs]

setup(name='ekg',
      version='0.4.0pre1',
      description='Collect and display statistics about open source projects',
      long_description=
'''EKG will scan such diverse sources as mailing lists, version control repositories, wikis to collect information about open source projects to determine the health of a community''',

      author='Yaakov M. Nemoy',
      author_email='ekg@hexago.nl',
      url='https://fedorahosted.org/ekg/',
      license='GPLv2+',

      packages=find_packages(),
      install_requires=['ConfigObj>=4.5.3', 'SQLAlchemy >= 0.5.2', 'python-dateutil',
                        'urlgrabber', 'sqlalchemy-migrate'],
      include_package_data=True,

      entry_points= {
        'console_scripts': command_funcs(commands),
        }
      )



from sqlalchemy import *
from migrate import *

meta = MetaData(migrate_engine)

mbox_sources = Table('mbox_sources', meta,
  Column('id', Integer(),  primary_key=True, nullable=False),
)

mailmen = Table('mailmen', meta,
  Column('id', Integer(),  primary_key=True, nullable=False),
)

mboxfacts = Table('mboxfacts', meta,
  Column('id', Integer(),  primary_key=True, nullable=False),
)

mboxmailmen = Table('mboxmailmen', meta,
  Column('id', Integer(),  primary_key=True, nullable=False),
)

fpmailmen = Table('fpmailmen', meta,
  Column('id', Integer(),  primary_key=True, nullable=False),
)

sources = Table('sources', meta,
  Column('id', Integer(),  primary_key=True, nullable=False),
  Column('type', Text(length=None, convert_unicode=False, assert_unicode=None),  nullable=False),
  Column('cache_file', Text(length=None, convert_unicode=False, assert_unicode=None)),
  Column('cache_url', Text(length=None, convert_unicode=False, assert_unicode=None)),
  Column('size', Integer()),
  Column('identifier', Text(length=None, convert_unicode=False, assert_unicode=None)),
  Column('month', DateTime(timezone=False)),
  Column('stream_id', Integer()),
)

fhmailman = Table('fhmailman', meta,
  Column('id', Integer(),  primary_key=True, nullable=False),
)

facts = Table('facts', meta,
  Column('id', Integer(),  primary_key=True, nullable=False),
  Column('type', Text(length=None, convert_unicode=False, assert_unicode=None),  nullable=False),
  Column('sender_name', Text(length=None, convert_unicode=False, assert_unicode=None)),
  Column('sender_handle', Text(length=None, convert_unicode=False, assert_unicode=None)),
  Column('sender', Text(length=None, convert_unicode=False, assert_unicode=None)),
  Column('sender_domain', Text(length=None, convert_unicode=False, assert_unicode=None)),
  Column('message_id', Text(length=None, convert_unicode=False, assert_unicode=None)),
  Column('date', DateTime(timezone=False)),
  Column('subject', Text(length=None, convert_unicode=False, assert_unicode=None)),
  Column('source_id', Integer()),
)

streams = Table('streams', meta,
  Column('id', Integer(),  primary_key=True, nullable=False),
  Column('type', Text(length=None, convert_unicode=False, assert_unicode=None),  nullable=False),
  Column('name', Text(length=None, convert_unicode=False, assert_unicode=None)),
)

rhmailmainsources = Table('rhmailmainsources', meta,
  Column('id', Integer(),  primary_key=True, nullable=False),
)

rhmailmen = Table('rhmailmen', meta,
  Column('id', Integer(),  primary_key=True, nullable=False),
)

rhmailmanfacts = Table('rhmailmanfacts', meta,
  Column('id', Integer(),  primary_key=True, nullable=False),
)


def upgrade():
    meta.create_all(migrate_engine)

def downgrade():
    meta.drop_all(migrate_engine)
